/*-------------------------------------------------------------------------
 *
 * walsender.h
 *	  Exports from replication/walsender.c.
 *
 * Portions Copyright (c) 2010-2010, PostgreSQL Global Development Group
 *
 * $PostgreSQL: pgsql/src/include/replication/walsender.h,v 1.5 2010/09/11 15:48:04 heikki Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifndef _WALSENDER_H
#define _WALSENDER_H

#include "access/xlog.h"
#include "storage/latch.h"
#include "storage/spin.h"

/*
 * Values for WalSnd->walSndState.
 */
typedef enum
{
	WALSND_INIT,		/* launched and initialized */
	WALSND_ASYNC,		/* performing asynchronous replication */
	WALSND_CATCHUP,		/* bulk-sending WAL for standby to catch up */
	WALSND_PRESYNC,		/* sent all the WAL required to get into sync */
	WALSND_SYNC			/* in sync with standby */
} WalSndState;

/*
 * Each walsender has a WalSnd struct in shared memory.
 */
typedef struct WalSnd
{
	pid_t		pid;			/* this walsender's process id, or 0 */
	XLogRecPtr	sentPtr;		/* WAL has been sent up to this point */
	XLogRecPtr	ackdPtr;		/* WAL has been replicated up to this point */

	WalSndState	walSndState;	/* current state */

	slock_t		mutex;			/* locks shared variables shown above */

	/*
	 * Latch used by backends to wake up this walsender when it has work
	 * to do.
	 */
	Latch		latch;
} WalSnd;

/* There is one WalSndCtl struct for the whole database cluster */
typedef struct
{
	/* Protected by WalSndWaiterLock */
	int			numWaiters;	/* current # of WalSndWaiters */
	int			maxWaiters;	/* allocated size of WalSndWaiters */

	WalSnd		walsnds[1];		/* VARIABLE LENGTH ARRAY */
} WalSndCtlData;

extern WalSndCtlData *WalSndCtl;

/*
 * Each waiter has a WalSndWaiter struct in shared memory.
 */
typedef struct WalSndWaiter
{
	BackendId	backendId;	/* this waiter's backend ID */
	XLogRecPtr	record;		/* this waiter wants for replication to be
							 * acked up to this point */
	Latch	   *latch;		/* pointer to the latch used to wake up this
							 * waiter */
} WalSndWaiter;

/* global state */
extern bool am_walsender;

/* user-settable parameters */
extern int	WalSndDelay;
extern int	max_wal_senders;
extern int	replication_timeout;
extern char	*standby_fencing_command;

extern int	WalSenderMain(void);
extern void WalSndSignals(void);
extern Size WalSndShmemSize(void);
extern void WalSndShmemInit(void);
extern void WalSndWakeup(void);
extern void WaitXLogSend(XLogRecPtr record);

extern void HandleReplicationComplete(void);
extern void HandleExitBeforeComplete(void);
extern bool NeedExitBeforeComplete(void);

#endif   /* _WALSENDER_H */
